require "libs.matrix"
require "libs.triangle"
require "libs.vector"
require "libs.meshLoader"
require "src.camera"
require "src.mesh"

function love.load()
    lg = love.graphics
    SCREENWIDTH, SCREENHEIGHT = lg.getWidth(), lg.getHeight()
    MATRIXPROJECTION = Matrix.createProjection(90, 1000, 0.1, SCREENWIDTH , SCREENHEIGHT)
    MP_ARRAY = Matrix.makeArray(MATRIXPROJECTION)
    Camera.load()
    LIGHT_DIR = Vector.new(0, 0, 0)
    FPS = love.timer.getFPS()
    SHADER = love.graphics.newShader("shader/pixel.glsl", "shader/vert.glsl")
    Light = Vector.new(0, 10, 0)
    Cube = Mesh.new("meshes/enterprisesmooth.obj", 0, 0, 0, {0.2, 0.2, 0.2}, Light)
    testImg = lg.newImage("meshes/obey_avatar.png")
    love.graphics.setDepthMode("less", true)
    lg.setMeshCullMode("front")

end

function love.update(dt)
    DT = dt
    FPS = love.timer.getFPS()
    love.window.setTitle(FPS)
    Cube:updateMatrix()
    Camera.move(dt)
    Cube.light_source = Camera.pos
    Camera.matrix = Camera.updateMatrix(dt)
    if love.keyboard.isDown("g") then 
       love.mouse.setRelativeMode(false)
    end
end

function love.draw(dt)
    Cube:render()
end

function love.mousemoved(x, y, dirx, diry, touch )
   Camera.lookAround(dirx, diry)
end