require "libs.matrix"
require "libs.vector"
Camera = {}

function Camera.load()
    Camera.pos = Vector.new(0, 0, 0)
    Camera.pitchX = 0
    Camera.rollY = 0
    Camera.yawZ = 0
    Camera.target = Vector.new(0, 0, 1)
    Camera.up = Vector.new(0, 1, 0)
    Camera.lookDir = Vector.new(0, 0, -1)
    Camera.speed = 10
    Camera.sens = 0.5
    love.mouse.setRelativeMode(true)
    Camera.matrix = Camera.updateMatrix(0)
end


function Camera.setPosition(x, y, z)
    Camera.pos.x = x Camera.pos.y = y Camera.pos.z = z 
end

function  Camera.setRotation(x, y, z)
    Camera.pitchX = x Camera.rollY = y Camera.yawZ = z
end

function Camera.updateMatrix(dt)
    Camera.dt = dt
    MatCamRotY = Matrix.createRotationY(Camera.rollY)
    MatCamRotX = Matrix.createRotationX(Camera.pitchX)
    MatCamRotZ = Matrix.createRotationZ(Camera.yawZ)
    MatCamRotAll = Matrix.multiplyByMatrix(MatCamRotX, MatCamRotY)
    MatCamRotAll = Matrix.multiplyByMatrix(MatCamRotAll, MatCamRotZ)
    Camera.target = Vector.new(0, 0, 1)
    --Camera.up = Vector.new(0, 1, 0)
    --Camera.lookDir = Vector.new(0, 0, 1)
    Camera.lookDir = Vector.multiplyByMatrix(Camera.target, MatCamRotAll)
    Camera.target = Vector.add(Camera.pos,  Camera.lookDir)
    MatCamera = Matrix.pointAt(Camera.pos, Camera.target, Camera.up)
    MatView = Matrix.invert(MatCamera)
    return Matrix.makeArray(MatView)
end

function Camera.lookAround(dirx, diry)
    if diry > 0  then 
        Camera.pitchX = Camera.pitchX - diry * Camera.sens  * Camera.dt
    elseif diry < 0 then
        Camera.pitchX = Camera.pitchX - diry *  Camera.sens  * Camera.dt
    end
    if dirx < 0  then 
        Camera.rollY = Camera.rollY - dirx * Camera.sens * Camera.dt
    elseif dirx > 0 then
        Camera.rollY =  Camera.rollY - dirx * Camera.sens  * Camera.dt
    end


    --prevent the camera from looping back around/breaking your neck
    if Camera.pitchX > math.pi-0.1 then Camera.pitchX = math.pi-0.1 end
    if Camera.pitchX < -math.pi+0.1 then Camera.pitchX = -math.pi+0.1 end
end

function Camera.move(dt)
    DT = dt
    local forward = Vector.mul(Camera.lookDir, Camera.speed * dt)
    if love.keyboard.isDown("space") then 
        Camera.pos.y = Camera.pos.y + Camera.speed * dt
    end
    if love.keyboard.isDown("lshift") then 
        Camera.pos.y = Camera.pos.y - Camera.speed * dt

    end
    if love.keyboard.isDown("d") then 
        Camera.pos = Vector.subtract(Camera.pos, Vector.crossProduct(forward, Camera.up))
    end
    if love.keyboard.isDown("a") then 
        Camera.pos = Vector.add(Camera.pos, Vector.crossProduct(forward, Camera.up))
    end
    if love.keyboard.isDown("w") then 
        Camera.pos = Vector.add(Camera.pos, forward)
    end
    if love.keyboard.isDown("s") then 
        Camera.pos = Vector.subtract(Camera.pos, forward)
    end
end