LoadMesh = require "libs.g3dObjLoader"
require "libs.matrix"
Mesh = {}
Mesh.__index = Mesh
function Mesh.new(filename, x,y,z, ambilight, light_source)
    local ins = setmetatable(Mesh, {})
    ins.angle = {x = 0, y = 0, z = 0}
    ins.position = {x = x ,y = y ,z = z}
    ins.light_source = Vector.new(light_source.x, light_source.y, light_source.z, 1)
    ins.rotationMatrixX = Matrix.createRotationX(ins.angle.x)
    ins.rotationMatrixY = Matrix.createRotationY(ins.angle.y)
    ins.rotationMatrixZ = Matrix.createRotationZ(ins.angle.z)
    ins.translationMatrix = Matrix.createTranslate(ins.position.x, ins.position.y, ins.position.z)
    ins.ambientlight = Vector.new(ambilight[1], ambilight[2], ambilight[3])
    ins.ModelMatrix = {}
    ins.vertexFormat = {
        {"VertexPosition", "float", 3},
        {"VertexTexCoord", "float", 2},
        {"VertexNormal", "float", 3},
        {"VertexColor", "byte", 4},
    }
    ins.verts = LoadMesh(filename)
    ins.mesh = lg.newMesh(ins.vertexFormat, ins.verts, "triangles")
    return ins
end

function Mesh:updateMatrix()
    --multiply all Matrices and turn into shader friendly array
    self.rotationMatrixX = Matrix.createRotationX(self.angle.x)
    self.rotationMatrixY = Matrix.createRotationY(self.angle.y)
    self.rotationMatrixZ = Matrix.createRotationZ(self.angle.z)
    self.translationMatrix = Matrix.createTranslate(self.position.x, self.position.y, self.position.z)
    local mm = Matrix.multiplyByMatrix(self.rotationMatrixX, self.rotationMatrixY)
    local mm2 = Matrix.multiplyByMatrix(mm, self.rotationMatrixZ)
    self.ModelMatrix = Matrix.makeArray( Matrix.multiplyByMatrix(mm2, self.translationMatrix))
end

function Mesh:setX(x) self.position.x = x end
function Mesh:setY(y) self.position.y = y end
function Mesh:setZ(z) self.position.z = z end


function Mesh:render()
    lg.setShader(SHADER)
    SHADER:send("ModelMatrix", self.ModelMatrix)
    SHADER:send("ViewMatrix", Camera.matrix)
    SHADER:send("MATRIXPROJECTION", MP_ARRAY)
    SHADER:send("LightColor", {1, 1, 1, 1})-- {249.0/255.0, 215.0/255.0, 28.0/255.0, 1})
    local lightPos = {self.light_source.x, self.light_source.y, self.light_source.z, 1}
    SHADER:send("LightSource", lightPos)
    -- SHADER:send("TIME", love.timer.getTime())
    --SHADER:send("CAMERAPOS", Camera.pos)
    lg.setColor(1, 1, 1, 1)
    --self.mesh:setTexture(testImg)
    --lg.setWireframe(true)
    lg.draw(self.mesh)
end